@@ -4,10 +4,11 @@
     @E-mail : pan_jinquan@163.com
     @Date   : 2021-07-28 09:09:32
 """
-from tqdm import tqdm
-from ..engine import base
+
 import torch
 import torch.utils.data as torch_utils
+from tqdm import tqdm
+from ..engine import base
 
 
 class Engine(base.Base):
@@ -72,12 +73,25 @@ class Engine(base.Base):
             callback.on_test_summary(inputs, outputs, losses, epoch, batch, logs)
 
     def _device(self, inputs):
-        if isinstance(inputs, tuple) or isinstance(inputs, list):
-            inputs = [v.to(self.device) if isinstance(v, torch.Tensor) else v for v in inputs]
+        if isinstance(inputs, torch.Tensor):
+            inputs = inputs.to(self.device)
+        elif isinstance(inputs, tuple):
+            inputs = (self._device(v) for v in inputs)
+        elif isinstance(inputs, list):
+            inputs = [self._device(v) for v in inputs]
         elif isinstance(inputs, dict):
-            inputs = {k: v.to(self.device) if isinstance(v, torch.Tensor) else v for k, v in inputs.items()}
+            inputs = {k: self._device(v) for k, v in inputs.items()}
         return inputs
 
+    def _losses(self, losses):
+        if isinstance(losses, dict):
+            l = sum(losses.values())
+        elif isinstance(losses, list):
+            l = sum(losses)
+        else:
+            l = losses
+        return l
+
     def run(self, logs: dict = {}):
         self.model.to(self.device)
         self.on_train_begin(logs)
@@ -104,7 +118,7 @@ class Engine(base.Base):
                 outputs, losses = self.run_step(inputs)
                 # compute gradient and do optimizer step
                 self.optimizer.zero_grad()
-                losses.backward()
+                self._losses(losses).backward()
                 self.optimizer.step()
                 logs['lr'] = self.optimizer.param_groups[0]["lr"]
                 self.on_train_summary(inputs, outputs, losses, self.epoch, batch, logs)
