@@ -73,7 +73,7 @@ def load_config(config_file='config.yaml'):
 
 def save_config(data, config_file='config.yaml'):
     """保存yaml文件"""
-    fw = open(config_file, 'a', encoding='utf-8')
+    fw = open(config_file, 'w', encoding='utf-8')
     yaml.dump(data, fw)
     return config_file
 
